/*
 * Copyright: (C) 1999-2000 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */
#include <stdlib.h>
#include "aflibAudioStereoToMono.h"


/*! \brief Constructor for type of mixing operation.

   This constructor allows one to mix any input to a mono signal. It will mix both
   channels (aflibAudioStereoToMono::AFLIB_MIX_BOTH), or channel 1
   (aflibAudioStereoToMono::AFLIB_MIX_CHAN1), or channel 2
   (aflibAudioStereoToMono::AFLIB_MIX_CHAN2).
*/
aflibAudioStereoToMono::aflibAudioStereoToMono(
   aflibAudio& audio,
   aflibAudioStereoToMono::aflib_mix_type mix) : aflibAudioMixer(audio)
{
   setMix(mix);
}


/*! \brief Destructor.
*/
aflibAudioStereoToMono::~aflibAudioStereoToMono()
{
}


/*! \brief Sets the type of mixing to perform.

   This function allows one to change the type of mixing performed. One can select
   both channels (aflibAudioStereoToMono::AFLIB_MIX_BOTH), or channel 1
   (aflibAudioStereoToMono::AFLIB_MIX_CHAN1), or channel 2
   (aflibAudioStereoToMono::AFLIB_MIX_CHAN2).
*/
void
aflibAudioStereoToMono::setMix(aflibAudioStereoToMono::aflib_mix_type mix)
{
   _mix = mix;

   // Cause chain to be reprocessed to setInputConfig gets called again with
   // correct info.
   setNodeProcessed(FALSE);
}


void
aflibAudioStereoToMono::setInputConfig(const aflibConfig& cfg)
{
   map<int, aflibAudio *, less<int> > parents;
   int   id;

   // Get the ID number of the first input. We will only use this input
   parents = getParents();
   id = (*parents.begin()).first;

   // Delete all current mixing info.
   delAllMix();

   // Program mixing depending on state of input
   if (cfg.getChannels() >= 2)
   {
      if (_mix == AFLIB_MIX_BOTH)
      { 
         addMix(id, 0, 0, 50);
         addMix(id, 1, 0, 50);
      }
      else if (_mix == AFLIB_MIX_CHAN1)
      {
         addMix(id, 0, 0, 100);
      }
      else if (_mix == AFLIB_MIX_CHAN2)
      {
         addMix(id, 1, 0, 100);
      }
   }
   else
   {
      // If only a single channel input then just output that channel
      addMix(id, 0, 0, 100);
   }

   // Now let the base class compute the mixing information
   aflibAudioMixer::setInputConfig(cfg);
}



