/*
 * Copyright: (C) 1999-2000 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */

#include "aflibInfo.h"
#include "aflibAudio.h"
#include "aflibConfig.h"

#include <string>
#include <iostream>
/*! \class aflibInfo
    \brief Print info to stderr.

 This class just makes it easier to print what we need to. 
*/

static char* aflib_error_string[] =
{
   "AFLIB_SUCCESS",
   "AFLIB_ERROR_OPEN",
   "AFLIB_ERROR_UNSUPPORTED",
   "AFLIB_ERROR_INITIALIZATION_FAILURE",
   "AFLIB_NOT_FOUND",
   "AFLIB_END_OF_FILE",
   "AFLIB_NO_DATA",
	0
};

static char* aflib_data_size_string[] =   
{
   "UNDEFINED",
   "8 bit signed",
   "8 bit unsigned",
   "16 bit signed",
   "16 bit unsigned",
	0
};

static char* aflib_data_endian_string[] =
{
   "UNDEFINED",
   "ENDIAN_LITTLE",
   "ENDIAN_BIG",
	0
};

/*
const char *aflib_sizes_str[] = {
	"NONSENSE!",
	"8-bit",
	"16-bit",
	"NONSENSE",
	"32-bit",
	"32-bit floats",
	"64-bit floats",
	"IEEE floats"
};

const char *aflib_encodings_str[] = {
	"NONSENSE!",
	"unsigned",
	"signed (2's complement)",
	"u-law",
	"a-law",
	"adpcm",
	"ima_adpcm",
	"gsm",
};
*/

void
aflibInfo::printInfo(aflibAudio& audio)
{
	aflibConfig config = audio.getInputConfig();
	cerr << "Format: " << audio.getName() << endl;
	cerr << "------------Input Config------------" << endl;
	printConfig(config);
	config = audio.getOutputConfig();
	cerr << "------------Output Config-----------" << endl;
	printConfig(config);

}

void
aflibInfo::printConfig(aflibConfig& config)
{
   cerr << "Sample Rate: " << config.getSamplesPerSecond() << endl;
   cerr << "Num Channels: " << config.getChannels() << endl;
   cerr << "Num Samples: " << config.getTotalSamples() << endl;
	cerr << aflib_data_size_string[config.getSampleSize()] << endl;
	cerr << aflib_data_endian_string[config.getDataEndian()] << endl;

}

void
aflibInfo::printError(aflibStatus& status)
{
   cerr << "AFLIB ERROR: ";
	cerr << aflib_error_string[status] << endl;
}

