/*
 * Copyright: (C) 2000 Bruce W. Forsberg
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 *
 *   Bruce Forsberg  forsberg@adnc.com
 *
 */

// class for WAV audio file reading and writing

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/types.h>		/* for fstat() */
#include <sys/stat.h>		/* for fstat() */
#include <string.h>			/* strncmp() and memcpy() */

#include "aflibSoxFile.h"
#include "aflibFileItem.h"
#include "aflibData.h"
//#include "aflibInfo.h"

#define MODULE_NAME "aflibSoxFile"


typedef struct aflib_format {
	string mod_format;
	string mod_desc;
	string mod_ext;
	string mod_value1;
	string mod_value2;
	string mod_value3;
	string mod_magic;
}aflib_format_t;


extern "C"
{
   aflibFile *
   getAFileObject() { return ((aflibFile *)new aflibSoxFile()); }
/*
void
query_options( map<string, aflibOptions* >& options_map )
{
*/

/*
enum {
	OPTION_ENCODING,
	OPTION_SIZE
};
*/
/*
char* encoding_list[] = { 
		"unsigned","signed","U-law","A-law","adpcm","ima","gsm",(char*)NULL };
char* size_list[] = { 
		"byte","word","long","float","double","ieee",(char*)NULL };
*/
// How do I init a list sometimes C++ sucks
/*
_toptions sox_option[] = {
	{ 't',AFLIB_TYPE_CHAR | AFLIB_SELECT_LIST,"Encoding","Encoding Type",
					},
	{ 's',AFLIB_TYPE_CHAR | AFLIB_SELECT_LIST,"Data Size","Data Size",
									}
};
*/
/*	
	aflibOptions* options = new aflibOptions();
	_toptions option;
	option.id = 't';
	option.flags = AFLIB_TYPE_CHAR | AFLIB_SELECT_LIST;
	option.label = "Encoding";
	option.help = "Encoding Type";

	for(int i = 0; encoding_list[i]; i++){
		string opt = encoding_list[i];
		option.optlist.push_back(opt);
	}

	options->insertOption(option);


	option.id = 's';
	option.flags = AFLIB_TYPE_CHAR | AFLIB_SELECT_LIST;
	option.label = "Data Size";
	option.help = "Data Size";
	option.optlist.clear();
	for(int i = 0; size_list[i]; i++){
		string opt = size_list[i];
		option.optlist.push_back(opt);
	}

	options->insertOption(option);
	options_map["AU"] = options;
	options_map["WAV"] = options;
	options_map["AIFF"] = options;
	options_map["AIFF-C"] = options;

}
*/
void
query(list<aflibFileItem*>& support_list)
{

aflib_format_t formats[] = {

{
"8SVX","Amiga 8svx File Format",".svx","8svx","NONE","NONE",
"0(F), 1(O), 2(R), 3(M), 8(8), 9(S), 10(V), 11(X)" },

{ 
"AIFF","Audio Interchange File Format",".aiff","aiff","NONE","NONE",
"0(F), 1(O), 2(R), 3(M), 8(A), 9(I), 10(F), 11(F)" },

{ 
"AIFF-ULAW","Audio Interchange File Format - U-Law Compression",".aiff","aiff","ulaw","NONE",
"0(F), 1(O), 2(R), 3(M), 8(A), 9(I), 10(F), 11(F)" },

{ 
"AIFF-ALAW","Audio Interchange File Format - A-Law Compression",".aiff","aiff","alaw","NONE",
"0(F), 1(O), 2(R), 3(M), 8(A), 9(I), 10(F), 11(F)" },

{
"AIFC","Audio Interchange File Format",".aifc","aiff","NONE","NONE",
"0(F), 1(O), 2(R), 3(M), 8(A), 9(I), 10(F), 11(C)" },

{
"AU","NeXT .snd/Sun .au Format","au","au","NONE","NONE",
"0(.), 1(s), 2(n), 3(d)" },

{
"AU-ULAW","NeXT .snd/Sun .au Format U-Law Compression","au","au","ulaw","NONE",
"0(.), 1(s), 2(n), 3(d)" },

{
"AU-ALAW","NeXT .snd/Sun .au Format A-Law Compression","au","au","alaw","NONE",
"0(.), 1(s), 2(n), 3(d)" },

{
"AVR","Audio Visual Research File",".avr","avr","NONE","NONE",
"0(2), 1(B), 2(I), 3(T)" },

{ 
"MAUD","An Amiga Format",".maud","maud","NONE","NONE",
"0(F), 1(O), 2(R), 3(M), 8(M), 9(A), 10(U), 11(D)" },

{ 
"MAUD-ULAW","An Amiga Format",".maud","maud","ulaw","NONE",
"0(F), 1(O), 2(R), 3(M), 8(M), 9(A), 10(U), 11(D)" },

{ 
"MAUD-ALAW","An Amiga Format",".maud","maud","alaw","NONE",
"0(F), 1(O), 2(R), 3(M), 8(M), 9(A), 10(U), 11(D)" },

{
"NIST","NIST Sphere File Format",".sph","sph","NONE","NONE",
"0(N), 1(I), 2(S), 3(T), 4(_), 5(1), 6(A)" },

{
"SMP","Sound Tools SampleVision File Format",".smp","smp","NONE","NONE",
"0(S), 1(O), 2(U), 3(N), 4(D), 5( ), 6(S), 7(A), 8(M), 9(P), 10(L), 11(E), 12( ), 13(D), 14(A), 15(T), 16(A)" },

{
"SOUND","Sounder/Sndtool File Format",".sndt","sndt","NONE","NONE",
"0(S), 1(O), 2(U), 3(N), 4(D)" },

{
"TX-16W","Yamaha TX-16W sampler File Format",".txw","txw","NONE","NONE",
"0(L), 1(M), 2(8), 3(9), 4(5), 5(3)" },

{
"VOC","Creative Voice File",".voc","voc","NONE","NONE",
"0(C), 1(r), 2(e), 3(a), 4(t), 5(i), 6(v), 7(e)" },

{
"WAV","Microsoft Wave Format",".wav","wav","NONE","NONE",
"0(R), 1(I), 2(F), 3(F), 8(W), 9(A), 10(V), 11(E)"},

{
"WAV-ADPCM","Microsoft Wave Format - MS ADPCM Compression",".wav","wav","adpcm","NONE",
"0(R), 1(I), 2(F), 3(F), 8(W), 9(A), 10(V), 11(E)"},

{
"WAV-IMA","Microsoft Wave Format - IMA ADPCM Compression",".wav","wav","ima","NONE",
"0(R), 1(I), 2(F), 3(F), 8(W), 9(A), 10(V), 11(E)"},

{
"WAV-ULAW","Microsoft Wave Format - U-Law Compression",".wav","wav","ulaw","NONE",
"0(R), 1(I), 2(F), 3(F), 8(W), 9(A), 10(V), 11(E)"},
{
"WAV-ALAW","Microsoft Wave Format - A-Law Compression",".wav","wav","alaw","NONE",
"0(R), 1(I), 2(F), 3(F), 8(W), 9(A), 10(V), 11(E)"},
{
"WVE","Psion wve File Format",".wve","wve","NONE","NONE",
"0(A), 1(L), 2(a), 3(w), 4(S), 5(o), 6(u), 7(n), 8(d), 9(F), 10(i), 11(l), 12(e), 13(*), 14(*)" },

{"ALSA","Linux Audio Device","NONE","alsa","NONE","NONE","NONE" },

{"CDR","CD audio",".cdr","cdr","NONE","NONE","NONE" },

{
"CVSD","Continuously Varible Slope Delta modulation",".cvs","cvs","NONE","NONE",
"NONE" },

{"DEVICE","Linux Audio Device","NONE","ossdsp","NONE","NONE","NONE" },

{
"VMS","More info to come",".vms","vms","NONE","NONE",
"NONE" },

// Better way to do this one? 
{"NONE","NONE","NONE","NONE","NONE","NONE","NONE"}

};
	aflibFileItem* item;
	for(aflib_format_t* format=&formats[0];format->mod_format !="NONE";format++){			
		//aflib_format_t* format = &formats[0];
		item	= new aflibFileItem();

		item->setFormat(format->mod_format);
		item->setDescription(format->mod_desc);
		item->setExtension(format->mod_ext);
		item->setName(MODULE_NAME);
		if(format->mod_magic != "NONE" )
			item->setMagic(format->mod_magic);
		item->setValue1(format->mod_value1);
		item->setValue2(format->mod_value2);
		item->setValue3(format->mod_value3);
		support_list.push_back(item);
	}
	
}

}

aflibSoxFile::aflibSoxFile()

{

//	aflibOptions options;

	_st_format.info.rate      = 0;
	_st_format.info.size      = -1;
	_st_format.info.encoding     = -1;
	_st_format.info.channels  = -1;
	_st_format.comment   = NULL;
	_st_format.seekable	= 0;
	_st_format.swap      = 0;
	_st_format.filetype  = (char *) 0;
	_st_format.fp        = NULL;
	_st_format.filename  = NULL;
	_st_format.length = 0;
	_write = FALSE;
	_position = 0;
//	setOptionStrings(options);
//	setOptions(options);
	
}

aflibSoxFile::~aflibSoxFile()
{

   if (_st_format.fp)
   {
    	if(_write)
           if (_st_format.h)
              _st_format.h->stopwrite(&_st_format);
	else
           if (_st_format.h)
              _st_format.h->stopread(&_st_format);
    	fclose(_st_format.fp);
   }
}
/*
void
aflibSoxFile::initOptions(aflibOptions& options)
{
	int value = 0;
	
	value = _st_format.info.encoding - 1;
//	options.setOption('t',(char*)&value);
	value = _st_format.info.size < 3 ? _st_format.info.size - 1 :
			_st_format.info.size - 2;
//	options.setOption('s',(char*)&value);
}

void
aflibSoxFile::setOptionStrings(aflibOptions& options)
{

	options.insertOption(sox_option[OPTION_ENCODING]);
	options.insertOption(sox_option[OPTION_SIZE]);

}
*/
aflibStatus
aflibSoxFile::afopen(
   const char * file,
   aflibConfig* cfg)
{

	aflibStatus  status = AFLIB_SUCCESS;
  	aflibConfig  input_cfg(*cfg);
   aflibData    data(1);
	// configToFormat(input_cfg,&_st_format);

//	aflibOptions options = getOptions();

        _st_format.file.eof = 0;
	_st_format.filetype = (char*)_value1.c_str();
	_st_format.filename = (char*)calloc(strlen(file)+1,1);
	strcpy(_st_format.filename,file);

	_st_format.fp = fopen(_st_format.filename,READBINARY);
  	if (!_st_format.fp) return (AFLIB_ERROR_OPEN);

/* from Sox */
#if	defined(DUMB_FILESYSETM)
	_st_format.seekable  = 0;
#else
	_st_format.seekable  = (filetype(fileno(_st_format.fp)) == S_IFREG);
#endif

	if ( st_gettype(&_st_format) ) return(AFLIB_ERROR_OPEN);

    /* Read and write starters can change their formats. */
   if(_st_format.h->startread(&_st_format)) return(AFLIB_ERROR_OPEN);

   if ( st_checkformat(&_st_format) )  return(AFLIB_ERROR_OPEN);

   // Set input and output audio configuration
	formatToConfig(input_cfg,&_st_format);
  	setInputConfig(input_cfg);
  	setOutputConfig(input_cfg);

//	initOptions(options);
//	setOptions(options);

  	return (status);
}

aflibStatus
aflibSoxFile::afcreate(
   const char * file,
   const aflibConfig& cfg)
{
  	aflibStatus  status = AFLIB_SUCCESS;
  	aflibConfig  output_cfg(cfg);

  	(void)unlink(file);

	configToFormat(output_cfg,&_st_format);
	_st_format.filename = (char*)calloc(strlen(file)+1,1);
	strcpy(_st_format.filename,file);

	_st_format.fp = fopen(_st_format.filename,WRITEBINARY);
  	if (!_st_format.fp) return(AFLIB_ERROR_OPEN);

	_st_format.filetype = (char*)_value1.c_str();
	_st_format.comment = _st_format.filename;
	if(_value2 == "ulaw") _st_format.info.encoding = ST_ENCODING_ULAW;
	else if(_value2 == "alaw") _st_format.info.encoding = ST_ENCODING_ALAW;
	else if(_value2 == "adpcm") _st_format.info.encoding = ST_ENCODING_ADPCM;
	else if(_value2 == "ima") _st_format.info.encoding = ST_ENCODING_IMA_ADPCM;

	if( st_gettype(&_st_format)) return(AFLIB_ERROR_OPEN);

/* from Sox */
#if	defined(DUMB_FILESYSETM)
	_st_format.seekable  = 0;
#else
	_st_format.seekable  = (filetype(fileno(_st_format.fp)) == S_IFREG);
#endif

    /* Read and write starters can change their formats. */
    if(_st_format.h->startwrite(&_st_format)) return(AFLIB_ERROR_OPEN);

    if (st_checkformat(&_st_format)) return(AFLIB_ERROR_OPEN);

   // Set input and output audio configuration
	formatToConfig(output_cfg,&_st_format);
	_write = TRUE;
  	setInputConfig(output_cfg);
  	setOutputConfig(output_cfg);

  	return(status);
}

aflibStatus
aflibSoxFile::afread(
   aflibData& data,
   long long position )
{
  	aflibStatus  status = AFLIB_SUCCESS;

  	char* p_data;
  	LONG   new_length;

/*  Allocate data object with info from config */
	data.setConfig(getInputConfig());
  	p_data = (char*)data.getDataPointer();

/*Return EOF if position out of range */
  	if (position >= _st_format.length/_st_format.info.channels)
  	{
     	data.adjustLength(0);
  		return(AFLIB_END_OF_FILE);
  	}

   // Seek to correct position
	if( position != -1 ){
		if(_st_format.h->seek(&_st_format,position*_st_format.info.channels)==0)
			_position = position*_st_format.info.channels;
	}
	new_length = data.getLength()*_st_format.info.channels;
   // Don't read past total samples available
	new_length = _st_format.length - _position < new_length ? 
			_st_format.length - _position : new_length;

// Convert from LONG to datasize

#ifndef NO_CONVERT
  	if (new_length){
		LONG datum;
		LONG done = 0;
	switch(_st_format.info.size) {
		case ST_SIZE_BYTE:
		    switch(_st_format.info.encoding)
		    {
			case ST_ENCODING_SIGN2:
				while(done < new_length) {
					/* scale signed up to long's range */
					_st_format.h->read(&_st_format,&datum,1);
					if(_st_format.file.eof) break;
					*p_data++ = (signed char) RIGHT(datum, 24);
					done++;
				}
				break;
			case ST_ENCODING_ALAW:
			case ST_ENCODING_ULAW:
			case ST_ENCODING_UNSIGNED:
				while(done < new_length) {
					_st_format.h->read(&_st_format,&datum,1);
					if(_st_format.file.eof) break;
					/* scale signed up to long's range */
					*p_data++ = (unsigned char)( RIGHT(datum, 24)^128);
					/* Convert to unsigned */
					done++;
				}
				break;
		    }
		    break;
		case ST_SIZE_WORD:
		    switch(_st_format.info.encoding)
		    {
			case ST_ENCODING_ALAW:
			case ST_ENCODING_ULAW:
			case ST_ENCODING_SIGN2:
			case ST_ENCODING_ADPCM:
				while(done < new_length) {
					_st_format.h->read(&_st_format,&datum,1);
					if(_st_format.file.eof) break;
					*p_data++ = (unsigned char)RIGHT(datum,16);
					*p_data++ = (unsigned char)RIGHT(datum,24);
					done++;
				}
				break;
			case ST_ENCODING_UNSIGNED:
				while(done < new_length) {
					_st_format.h->read(&_st_format,&datum,1);
					if(_st_format.file.eof) break;
					/* scale signed up to long's range */
					datum = RIGHT(datum,16)^ 0x8000;
					*p_data++ = (unsigned char)RIGHT(datum,16);
					*p_data++ = (unsigned char)RIGHT(datum,24);
					/* Convert to unsigned */
					done++;
				}
				break;
		    }
		    break;
		default:
			break;
	}
}
#else
// End

  	if (new_length)
		new_length = _st_format.h->read(&_st_format,p_data,new_length);
#endif

   // IF we reached the end of the file then return error
	// premature EOF.
   // IF we read the last chunk
  	if (new_length != data.getLength()*_st_format.info.channels)
  	{
      // When we only have a partial read then readjust the length of data
     	data.adjustLength(new_length/_st_format.info.channels);
  	}
  	if (_st_format.file.eof)
  	{
     	status = AFLIB_END_OF_FILE;
  	}
	_position += new_length;

  	return(status);
}

aflibStatus
aflibSoxFile::afwrite(
   aflibData& data,
   long long position )
{
  	aflibStatus  status = AFLIB_SUCCESS;

  	char* p_data;
  	LONG  new_length;

  	p_data = (char*)data.getDataPointer();
	new_length = data.getLength()*_st_format.info.channels;

	if(new_length == 0) return(status);

#ifndef NO_CONVERT

	LONG datum;
	LONG done = 0;
	switch(_st_format.info.size) {
		case ST_SIZE_BYTE:
		    switch(_st_format.info.encoding)
		    {
			case ST_ENCODING_SIGN2:
				while(done < new_length) {
					/* scale signed up to long's range */
					datum = LEFT(*p_data++, 24);
   					_st_format.h->write(&_st_format, &datum, 1);
					done++;
				}
				break;
			case ST_ENCODING_ULAW:
			case ST_ENCODING_UNSIGNED:
			case ST_ENCODING_ALAW:
				while(done < new_length) {
					datum = *p_data++;
					/* Convert to signed */
					datum ^= 128;
					/* scale signed up to long's range */
					datum = LEFT(datum, 24);
   					_st_format.h->write(&_st_format, &datum, 1);
					done++;
				}
				break;
		    }
		    break;
		case ST_SIZE_WORD:
		    switch(_st_format.info.encoding)
		    {
			case ST_ENCODING_SIGN2:
				while(done < new_length) {
					datum = LEFT(*(signed short *)p_data,16);
   					_st_format.h->write(&_st_format, &datum, 1);
					done++;
					p_data += 2;
				}
				break;
				
			case ST_ENCODING_ULAW:
			case ST_ENCODING_ALAW:
			case ST_ENCODING_UNSIGNED:
			case ST_ENCODING_ADPCM:
				while(done < new_length) {
					datum = *(unsigned short*) p_data;
					/* Convert to signed */
					datum ^= 0x8000;
					/* scale signed up to long's range */
					datum = LEFT(datum, 16);
   					_st_format.h->write(&_st_format, &datum, 1);
					done++;
					p_data += 2;
				}
				break;
		}
		default:
			break;
	}
#else
	new_length = _st_format.h->write(&_st_format, p_data, new_length);
#endif
 
/* Out of Disk Space ? */
   if (new_length < data.getLength()*_st_format.info.channels)
   {
      status = AFLIB_END_OF_FILE;
   }

   return(status);
}

bool
aflibSoxFile::isDataSizeSupported(aflib_data_size size)
{
   // If handle not yet allocated then indicate sizes supported

   if (_st_format.fp == NULL){ 
	if(_value2 == "ADPCM")
		return	(size==AFLIB_DATA_16S);
	else if(_value2 == "IMA_ADPCM")
		return	(size==AFLIB_DATA_16S);
// Need 16 bit signed to write 8 bit ALAW or ULAW 
	else if(_value2 == "ALAW")
		return	(size==AFLIB_DATA_16S);
	else if(_value2 == "ULAW")
		return	(size==AFLIB_DATA_16S);

	else if(_value1 == "ossdsp")
		return(	(size==AFLIB_DATA_8U)||
					(size==AFLIB_DATA_16S));
	else if(_value1 == "voc")
		return(	(size==AFLIB_DATA_8U)||
					(size==AFLIB_DATA_16S));
	else if(_value1 == "wav")
		return(	(size==AFLIB_DATA_8U)||
					(size==AFLIB_DATA_16S));
	else if(_value1 == "aiff")
		return(	(size==AFLIB_DATA_8S)||
					(size==AFLIB_DATA_16S));
	else if(_value1 == "cdr")
		return(	size==AFLIB_DATA_16S);
	else if(_value1 == "au")
		return(	(size==AFLIB_DATA_8S)||
					(size==AFLIB_DATA_16S));
	else if(_value1 == "svx")
		return	(size==AFLIB_DATA_8S);
	else
   	return ( TRUE );
	}
	else
	{
		return (size==getOutputConfig().getSampleSize());
	}

}
 
bool
aflibSoxFile::isEndianSupported(aflib_data_endian end)
{
   // If handle not yet allocated then indicate any endian is supported
   if (!_st_format.fp) return (TRUE);

   // Get the endian of the data
   if (_st_format.swap) return(end = AFLIB_ENDIAN_BIG);
   return(end = AFLIB_ENDIAN_LITTLE);
}

bool
aflibSoxFile::isSampleRateSupported(int& rate)
{

   // If handle not yet allocated then indicate any sample rate is supported
   if (_st_format.fp == NULL) return (TRUE);

   // IF same rate then TRUE else return desired rate
   if (_st_format.info.rate == rate)
      return( rate = _st_format.info.rate );
   
   rate = _st_format.info.rate;
   return FALSE;

}

bool
aflibSoxFile::isChannelsSupported(int& channels)
{

   // If handle not yet allocated then indicate any sample rate is supported
   if (_st_format.fp == NULL) return (TRUE);

   // IF same rate then TRUE else return desired rate
   if (_st_format.info.channels == channels)
      return( TRUE );
   
   channels = _st_format.info.channels;
   return FALSE;

}

/*
bool 
aflibSoxFile::getOptions(map<string,string>& options){
	
	options["Data Size"] = st_sizes_str[_st_format.info.size];
	options["Encoding Style"] = st_encodings_str[_st_format.info.encoding];
	return 1;
}
*/
/*
bool 
aflibSoxFile::getOptions(map<string,string>& options){
	
	options["Data Size"] = st_sizes_str[_st_format.info.size];
	options["Encoding Style"] = st_encodings_str[_st_format.info.encoding];
	return 1;
}
*/
bool
aflibSoxFile::getItem( const char* item, void* value){

	if(!strncmp(item,"ft",2)){
		memcpy(value,&_st_format,sizeof(st_soundstream));
		return TRUE;
	} 
	return FALSE;
}

/* from Sox */
int aflibSoxFile::filetype(int fd)
{
	struct stat st;

	fstat(fd, &st);

	return st.st_mode & S_IFMT;
}

void
aflibSoxFile::configToFormat(
   const aflibConfig& cfg,
	ft_t ft
){
	ft->info.rate = cfg.getSamplesPerSecond();
	if(cfg.getChannels())
		ft->info.channels = cfg.getChannels();	

	switch(cfg.getSampleSize()){
		case AFLIB_DATA_8S :
			ft->info.encoding = ST_ENCODING_SIGN2;
			ft->info.size = ST_SIZE_BYTE;
		break;
		case AFLIB_DATA_8U :
			ft->info.encoding = ST_ENCODING_UNSIGNED;
			ft->info.size = ST_SIZE_BYTE;
		break;
		case AFLIB_DATA_16S :
			ft->info.encoding = ST_ENCODING_SIGN2;
			ft->info.size = ST_SIZE_WORD;
		break;
		case AFLIB_DATA_16U :
			ft->info.encoding = ST_ENCODING_UNSIGNED;
			ft->info.size = ST_SIZE_WORD;
		break;
/*		case AFLIB_DATA_32S :
			ft->info.encoding = ST_ENCODING_SIGN2;
			ft->info.size = ST_SIZE_DWORD;
		break;
*/
		case AFLIB_SIZE_UNDEFINED :
		break;
	}	
}

void
aflibSoxFile::formatToConfig(
   aflibConfig& cfg,
	const ft_t ft
){
	aflibData data(1);
   cfg.setDataOrientation(AFLIB_INTERLEAVE);

  	cfg.setDataEndian( data.getHostEndian() );
   cfg.setChannels(ft->info.channels);
   cfg.setSamplesPerSecond((int)ft->info.rate);
   cfg.setTotalSamples(ft->length/ft->info.channels);

   if (ft->info.size==ST_SIZE_BYTE )
   {
		switch(ft->info.encoding){
			case ST_ENCODING_UNSIGNED:
      		cfg.setSampleSize(AFLIB_DATA_8U);
			break;
			case ST_ENCODING_SIGN2:
      		cfg.setSampleSize(AFLIB_DATA_8S);
			break;
			case ST_ENCODING_ULAW:
			case ST_ENCODING_ALAW:
	// Ulaw and Alaw converters return signed 16 from bytes
	// see raw.c gXXX.c ...
      		cfg.setSampleSize(AFLIB_DATA_16S);
			break;
		}
   }
   else if (ft->info.size == ST_SIZE_WORD )
   {
		switch(ft->info.encoding){
			case ST_ENCODING_UNSIGNED:
      		cfg.setSampleSize(AFLIB_DATA_16U);
			break;
			case ST_ENCODING_SIGN2:
			case ST_ENCODING_ADPCM:
			case ST_ENCODING_IMA_ADPCM:
      		cfg.setSampleSize(AFLIB_DATA_16S);
			break;
		}
   }

}
